@@ -19,7 +19,7 @@ individual suites to be run::
     $ python -m examples.performance --help
     usage: python -m examples.performance [-h] [--test TEST] [--dburl DBURL]
                                           [--num NUM] [--profile] [--dump]
-                                          [--runsnake] [--echo]
+                                          [--echo]
 
                                           {bulk_inserts,large_resultsets,single_inserts}
 
@@ -35,7 +35,6 @@ individual suites to be run::
                             default is module-specific
       --profile             run profiling and dump call counts
       --dump                dump full call profile (implies --profile)
-      --runsnake            invoke runsnakerun (implies --profile)
       --echo                Echo SQL output
 
 An example run looks like::
@@ -107,15 +106,6 @@ individual tests::
 
         ...
 
-Using RunSnake
---------------
-
-This option requires the `RunSnake <https://pypi.python.org/pypi/RunSnakeRun>`_
-command line tool be installed::
-
-    $ python -m examples.performance single_inserts --test test_core --num 1000 --runsnake
-
-A graphical RunSnake output will be displayed.
 
 .. _examples_profiling_writeyourown:
 
@@ -213,13 +203,12 @@ We can run our new script directly::
     test_joinedload : load everything, joined eager loading. (1000 iterations); total time 2.754592 sec
     test_subqueryload : load everything, subquery eager loading. (1000 iterations); total time 2.977696 sec
 
-As well as see RunSnake output for an individual test::
-
-    $ python test_loads.py  --num 100 --runsnake --test test_joinedload
 
 """  # noqa
+
 import argparse
 import cProfile
+import gc
 import os
 import pstats
 import re
@@ -227,7 +216,7 @@ import sys
 import time
 
 
-class Profiler(object):
+class Profiler:
     tests = []
 
     _setup = None
@@ -238,12 +227,14 @@ class Profiler(object):
     def __init__(self, options):
         self.test = options.test
         self.dburl = options.dburl
-        self.runsnake = options.runsnake
         self.profile = options.profile
         self.dump = options.dump
+        self.raw = options.raw
         self.callers = options.callers
         self.num = options.num
         self.echo = options.echo
+        self.sort = options.sort
+        self.gc = options.gc
         self.stats = []
 
     @classmethod
@@ -278,9 +269,9 @@ class Profiler(object):
 
     def run(self):
         if self.test:
-            tests = [fn for fn in self.tests if fn.__name__ == self.test]
+            tests = [fn for fn in self.tests if fn.__name__ in self.test]
             if not tests:
-                raise ValueError("No such test: %s" % self.test)
+                raise ValueError("No such test(s): %s" % self.test)
         else:
             tests = self.tests
 
@@ -292,7 +283,7 @@ class Profiler(object):
             self._run_test(test)
             self.stats[-1].report()
 
-    def _run_with_profile(self, fn):
+    def _run_with_profile(self, fn, sort):
         pr = cProfile.Profile()
         pr.enable()
         try:
@@ -300,9 +291,9 @@ class Profiler(object):
         finally:
             pr.disable()
 
-        stats = pstats.Stats(pr).sort_stats("cumulative")
+        stats = pstats.Stats(pr)
 
-        self.stats.append(TestResult(self, fn, stats=stats))
+        self.stats.append(TestResult(self, fn, stats=stats, sort=sort))
         return result
 
     def _run_with_time(self, fn):
@@ -316,14 +307,18 @@ class Profiler(object):
     def _run_test(self, fn):
         if self._setup:
             self._setup(self.dburl, self.echo, self.num)
-        if self.profile or self.runsnake or self.dump:
-            self._run_with_profile(fn)
+        if self.gc:
+            # gc.set_debug(gc.DEBUG_COLLECTABLE)
+            gc.set_debug(gc.DEBUG_STATS)
+        if self.profile or self.dump:
+            self._run_with_profile(fn, self.sort)
         else:
             self._run_with_time(fn)
+        if self.gc:
+            gc.set_debug(0)
 
     @classmethod
     def main(cls):
-
         parser = argparse.ArgumentParser("python -m examples.performance")
 
         if cls.name is None:
@@ -338,7 +333,9 @@ class Profiler(object):
                 except ImportError:
                     pass
 
-        parser.add_argument("--test", type=str, help="run specific test name")
+        parser.add_argument(
+            "--test", nargs="+", type=str, help="run specific test(s)"
+        )
 
         parser.add_argument(
             "--dburl",
@@ -358,20 +355,29 @@ class Profiler(object):
             action="store_true",
             help="run profiling and dump call counts",
         )
+        parser.add_argument(
+            "--sort",
+            type=str,
+            default="cumulative",
+            help="profiling sort, defaults to cumulative",
+        )
         parser.add_argument(
             "--dump",
             action="store_true",
             help="dump full call profile (implies --profile)",
         )
+        parser.add_argument(
+            "--raw",
+            type=str,
+            help="dump raw profile data to file (implies --profile)",
+        )
         parser.add_argument(
             "--callers",
             action="store_true",
             help="print callers as well (implies --dump)",
         )
         parser.add_argument(
-            "--runsnake",
-            action="store_true",
-            help="invoke runsnakerun (implies --profile)",
+            "--gc", action="store_true", help="turn on GC debug stats"
         )
         parser.add_argument(
             "--echo", action="store_true", help="Echo SQL output"
@@ -379,7 +385,7 @@ class Profiler(object):
         args = parser.parse_args()
 
         args.dump = args.dump or args.callers
-        args.profile = args.profile or args.dump or args.runsnake
+        args.profile = args.profile or args.dump or args.raw
 
         if cls.name is None:
             __import__(__name__ + "." + args.name)
@@ -396,12 +402,15 @@ class Profiler(object):
         return suites
 
 
-class TestResult(object):
-    def __init__(self, profile, test, stats=None, total_time=None):
+class TestResult:
+    def __init__(
+        self, profile, test, stats=None, total_time=None, sort="cumulative"
+    ):
         self.profile = profile
         self.test = test
         self.stats = stats
         self.total_time = total_time
+        self.sort = sort
 
     def report(self):
         print(self._summary())
@@ -421,21 +430,16 @@ class TestResult(object):
         return summary
 
     def report_stats(self):
-        if self.profile.runsnake:
-            self._runsnake()
-        elif self.profile.dump:
-            self._dump()
+        if self.profile.dump:
+            self._dump(self.sort)
+        elif self.profile.raw:
+            self._dump_raw()
 
-    def _dump(self):
-        self.stats.sort_stats("time", "calls")
+    def _dump(self, sort):
+        self.stats.sort_stats(*re.split(r"[ ,]", self.sort))
         self.stats.print_stats()
         if self.profile.callers:
             self.stats.print_callers()
 
-    def _runsnake(self):
-        filename = "%s.profile" % self.test.__name__
-        try:
-            self.stats.dump_stats(filename)
-            os.system("runsnake %s" % filename)
-        finally:
-            os.remove(filename)
+    def _dump_raw(self):
+        self.stats.dump_stats(self.profile.raw)
