@@ -1,97 +1,72 @@
-# postgresql/json.py
-# Copyright (C) 2005-2023 the SQLAlchemy authors and contributors
+# dialects/postgresql/json.py
+# Copyright (C) 2005-2024 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: https://www.opensource.org/licenses/mit-license.php
-from __future__ import absolute_import
+# mypy: ignore-errors
 
+
+from .array import ARRAY
+from .array import array as _pg_array
+from .operators import ASTEXT
+from .operators import CONTAINED_BY
+from .operators import CONTAINS
+from .operators import DELETE_PATH
+from .operators import HAS_ALL
+from .operators import HAS_ANY
+from .operators import HAS_KEY
+from .operators import JSONPATH_ASTEXT
+from .operators import PATH_EXISTS
+from .operators import PATH_MATCH
 from ... import types as sqltypes
-from ... import util
-from ...sql import operators
-
+from ...sql import cast
 
 __all__ = ("JSON", "JSONB")
 
-idx_precedence = operators._PRECEDENCE[operators.json_getitem_op]
-
-ASTEXT = operators.custom_op(
-    "->>",
-    precedence=idx_precedence,
-    natural_self_precedent=True,
-    eager_grouping=True,
-)
-
-JSONPATH_ASTEXT = operators.custom_op(
-    "#>>",
-    precedence=idx_precedence,
-    natural_self_precedent=True,
-    eager_grouping=True,
-)
-
-
-HAS_KEY = operators.custom_op(
-    "?",
-    precedence=idx_precedence,
-    natural_self_precedent=True,
-    eager_grouping=True,
-)
-
-HAS_ALL = operators.custom_op(
-    "?&",
-    precedence=idx_precedence,
-    natural_self_precedent=True,
-    eager_grouping=True,
-)
-
-HAS_ANY = operators.custom_op(
-    "?|",
-    precedence=idx_precedence,
-    natural_self_precedent=True,
-    eager_grouping=True,
-)
-
-CONTAINS = operators.custom_op(
-    "@>",
-    precedence=idx_precedence,
-    natural_self_precedent=True,
-    eager_grouping=True,
-)
-
-CONTAINED_BY = operators.custom_op(
-    "<@",
-    precedence=idx_precedence,
-    natural_self_precedent=True,
-    eager_grouping=True,
-)
-
 
 class JSONPathType(sqltypes.JSON.JSONPathType):
-    def bind_processor(self, dialect):
-        super_proc = self.string_bind_processor(dialect)
-
+    def _processor(self, dialect, super_proc):
         def process(value):
-            assert isinstance(value, util.collections_abc.Sequence)
-            tokens = [util.text_type(elem) for elem in value]
-            value = "{%s}" % (", ".join(tokens))
+            if isinstance(value, str):
+                # If it's already a string assume that it's in json path
+                # format. This allows using cast with json paths literals
+                return value
+            elif value:
+                # If it's already a string assume that it's in json path
+                # format. This allows using cast with json paths literals
+                value = "{%s}" % (", ".join(map(str, value)))
+            else:
+                value = "{}"
             if super_proc:
                 value = super_proc(value)
             return value
 
         return process
 
+    def bind_processor(self, dialect):
+        return self._processor(dialect, self.string_bind_processor(dialect))
+
     def literal_processor(self, dialect):
-        super_proc = self.string_literal_processor(dialect)
+        return self._processor(dialect, self.string_literal_processor(dialect))
 
-        def process(value):
-            assert isinstance(value, util.collections_abc.Sequence)
-            tokens = [util.text_type(elem) for elem in value]
-            value = "{%s}" % (", ".join(tokens))
-            if super_proc:
-                value = super_proc(value)
-            return value
 
-        return process
+class JSONPATH(JSONPathType):
+    """JSON Path Type.
+
+    This is usually required to cast literal values to json path when using
+    json search like function, such as ``jsonb_path_query_array`` or
+    ``jsonb_path_exists``::
+
+        stmt = sa.select(
+            sa.func.jsonb_path_query_array(
+                table.c.jsonb_col, cast("$.address.id", JSONPATH)
+            )
+        )
+
+    """
+
+    __visit_name__ = "JSONPATH"
 
 
 class JSON(sqltypes.JSON):
@@ -143,12 +118,6 @@ class JSON(sqltypes.JSON):
 
         data_table.c.data[('key_1', 'key_2', 5, ..., 'key_n')].astext == 'some value'
 
-    .. versionchanged:: 1.1  The :meth:`_expression.ColumnElement.cast`
-       operator on
-       JSON objects now requires that the :attr:`.JSON.Comparator.astext`
-       modifier be called explicitly, if the cast works only from a textual
-       string.
-
     Index operations return an expression object whose type defaults to
     :class:`_types.JSON` by default,
     so that further JSON-oriented instructions
@@ -159,7 +128,7 @@ class JSON(sqltypes.JSON):
     using psycopg2, the DBAPI only allows serializers at the per-cursor
     or per-connection level.   E.g.::
 
-        engine = create_engine("postgresql://scott:tiger@localhost/test",
+        engine = create_engine("postgresql+psycopg2://scott:tiger@localhost/test",
                                 json_serializer=my_serialize_fn,
                                 json_deserializer=my_deserialize_fn
                         )
@@ -173,9 +142,6 @@ class JSON(sqltypes.JSON):
 
         :class:`_postgresql.JSONB`
 
-    .. versionchanged:: 1.1 :class:`_postgresql.JSON` is now a PostgreSQL-
-       specific specialization of the new :class:`_types.JSON` type.
-
     """  # noqa
 
     astext_type = sqltypes.Text()
@@ -189,10 +155,7 @@ class JSON(sqltypes.JSON):
          be used to persist a NULL value::
 
              from sqlalchemy import null
-             conn.execute(table.insert(), data=null())
-
-         .. versionchanged:: 0.9.8 - Added ``none_as_null``, and :func:`.null`
-            is now supported in order to persist a NULL value.
+             conn.execute(table.insert(), {"data": null()})
 
          .. seealso::
 
@@ -202,10 +165,8 @@ class JSON(sqltypes.JSON):
          :attr:`.JSON.Comparator.astext`
          accessor on indexed attributes.  Defaults to :class:`_types.Text`.
 
-         .. versionadded:: 1.1
-
         """
-        super(JSON, self).__init__(none_as_null=none_as_null)
+        super().__init__(none_as_null=none_as_null)
         if astext_type is not None:
             self.astext_type = astext_type
 
@@ -263,7 +224,10 @@ class JSONB(JSON):
     It also adds additional operators specific to JSONB, including
     :meth:`.JSONB.Comparator.has_key`, :meth:`.JSONB.Comparator.has_all`,
     :meth:`.JSONB.Comparator.has_any`, :meth:`.JSONB.Comparator.contains`,
-    and :meth:`.JSONB.Comparator.contained_by`.
+    :meth:`.JSONB.Comparator.contained_by`,
+    :meth:`.JSONB.Comparator.delete_path`,
+    :meth:`.JSONB.Comparator.path_exists` and
+    :meth:`.JSONB.Comparator.path_match`.
 
     Like the :class:`_types.JSON` type, the :class:`_postgresql.JSONB`
     type does not detect
@@ -280,8 +244,6 @@ class JSONB(JSON):
     in the same way that ``psycopg2.extras.register_default_json`` is used
     to register these handlers with the json type.
 
-    .. versionadded:: 0.9.7
-
     .. seealso::
 
         :class:`_types.JSON`
@@ -324,4 +286,40 @@ class JSONB(JSON):
                 CONTAINED_BY, other, result_type=sqltypes.Boolean
             )
 
+        def delete_path(self, array):
+            """JSONB expression. Deletes field or array element specified in
+            the argument array.
+
+            The input may be a list of strings that will be coerced to an
+            ``ARRAY`` or an instance of :meth:`_postgres.array`.
+
+            .. versionadded:: 2.0
+            """
+            if not isinstance(array, _pg_array):
+                array = _pg_array(array)
+            right_side = cast(array, ARRAY(sqltypes.TEXT))
+            return self.operate(DELETE_PATH, right_side, result_type=JSONB)
+
+        def path_exists(self, other):
+            """Boolean expression. Test for presence of item given by the
+            argument JSONPath expression.
+
+            .. versionadded:: 2.0
+            """
+            return self.operate(
+                PATH_EXISTS, other, result_type=sqltypes.Boolean
+            )
+
+        def path_match(self, other):
+            """Boolean expression. Test if JSONPath predicate given by the
+            argument JSONPath expression matches.
+
+            Only the first item of the result is taken into account.
+
+            .. versionadded:: 2.0
+            """
+            return self.operate(
+                PATH_MATCH, other, result_type=sqltypes.Boolean
+            )
+
     comparator_factory = Comparator
