@@ -1,10 +1,9 @@
-# coding: utf-8
-
 import datetime
 
 from sqlalchemy import bindparam
 from sqlalchemy import Column
 from sqlalchemy import DateTime
+from sqlalchemy import event
 from sqlalchemy import exc
 from sqlalchemy import func
 from sqlalchemy import Integer
@@ -24,7 +23,6 @@ from sqlalchemy.testing import is_
 from sqlalchemy.testing import mock
 from sqlalchemy.testing.assertions import AssertsCompiledSQL
 from .test_compiler import ReservedWordFixture
-from ...engine import test_deprecations
 
 
 class BackendDialectTest(
@@ -33,6 +31,35 @@ class BackendDialectTest(
     __backend__ = True
     __only_on__ = "mysql", "mariadb"
 
+    @testing.fixture
+    def mysql_version_dialect(self, testing_engine):
+        """yield a MySQL engine that will simulate a specific version.
+
+        patches out various methods to not fail
+
+        """
+        engine = testing_engine()
+        _server_version = [None]
+        with mock.patch.object(
+            engine.dialect,
+            "_get_server_version_info",
+            lambda conn: engine.dialect._parse_server_version(
+                _server_version[0]
+            ),
+        ), mock.patch.object(
+            engine.dialect, "_set_mariadb", lambda *arg: None
+        ), mock.patch.object(
+            engine.dialect,
+            "get_isolation_level",
+            lambda *arg: "REPEATABLE READ",
+        ):
+
+            def go(server_version):
+                _server_version[0] = server_version
+                return engine
+
+            yield go
+
     def test_reserved_words_mysql_vs_mariadb(
         self, mysql_mariadb_reserved_words
     ):
@@ -55,8 +82,6 @@ class BackendDialectTest(
         )
 
     def test_no_show_variables(self):
-        from sqlalchemy.testing import mock
-
         engine = engines.testing_engine()
 
         def my_execute(self, statement, *args, **kw):
@@ -75,8 +100,6 @@ class BackendDialectTest(
                 engine.connect()
 
     def test_no_default_isolation_level(self):
-        from sqlalchemy.testing import mock
-
         engine = engines.testing_engine()
 
         real_isolation_level = testing.db.dialect.get_isolation_level
@@ -100,11 +123,49 @@ class BackendDialectTest(
             ):
                 engine.connect()
 
+    @testing.combinations(
+        "10.5.12-MariaDB", "5.6.49", "5.0.2", argnames="server_version"
+    )
+    def test_variable_fetch(self, mysql_version_dialect, server_version):
+        """test #7518"""
+        engine = mysql_version_dialect(server_version)
+
+        fetches = []
+
+        # the initialize() connection does not seem to use engine-level events.
+        # not changing that here
+
+        @event.listens_for(engine, "do_execute_no_params")
+        @event.listens_for(engine, "do_execute")
+        def do_execute_no_params(cursor, statement, *arg):
+            if statement.startswith("SHOW VARIABLES") or statement.startswith(
+                "SELECT @@"
+            ):
+                fetches.append(statement)
+            return None
+
+        engine.connect()
+
+        if server_version == "5.0.2":
+            eq_(
+                fetches,
+                [
+                    "SHOW VARIABLES LIKE 'sql_mode'",
+                    "SHOW VARIABLES LIKE 'lower_case_table_names'",
+                ],
+            )
+        else:
+            eq_(
+                fetches,
+                ["SELECT @@sql_mode", "SELECT @@lower_case_table_names"],
+            )
+
     def test_autocommit_isolation_level(self):
         c = testing.db.connect().execution_options(
             isolation_level="AUTOCOMMIT"
         )
         assert c.exec_driver_sql("SELECT @@autocommit;").scalar()
+        c.rollback()
 
         c = c.execution_options(isolation_level="READ COMMITTED")
         assert not c.exec_driver_sql("SELECT @@autocommit;").scalar()
@@ -134,6 +195,10 @@ class DialectTest(fixtures.TestBase):
         (2006, "foo", "OperationalError", "pymysql", True),
         (2007, "foo", "OperationalError", "mysqldb", False),
         (2007, "foo", "OperationalError", "pymysql", False),
+        (4031, "foo", "OperationalError", "mysqldb", True),
+        (4031, "foo", "OperationalError", "pymysql", True),
+        (4032, "foo", "OperationalError", "mysqldb", False),
+        (4032, "foo", "OperationalError", "pymysql", False),
     )
     def test_is_disconnect(
         self, arg0, message, exc_cls_name, dialect_name, is_disconnect
@@ -154,11 +219,7 @@ class DialectTest(fixtures.TestBase):
         eq_(dialect.is_disconnect(error, None, None), is_disconnect)
 
     @testing.combinations(
-        ("mysqldb"),
-        ("pymysql"),
-        ("oursql"),
-        id_="s",
-        argnames="driver_name",
+        ("mysqldb"), ("pymysql"), id_="s", argnames="driver_name"
     )
     def test_ssl_arguments(self, driver_name):
         url = (
@@ -186,7 +247,6 @@ class DialectTest(fixtures.TestBase):
             expected["ssl"]["check_hostname"] = False
 
         kwarg = dialect.create_connect_args(make_url(url))[1]
-        # args that differ between oursql and others
         for k in ("use_unicode", "found_rows", "client_flag"):
             kwarg.pop(k, None)
         eq_(kwarg, expected)
@@ -207,7 +267,7 @@ class DialectTest(fixtures.TestBase):
         dialect = mysqldb.dialect()
         connect_args = dialect.create_connect_args(
             make_url(
-                "mysql://scott:tiger@localhost:3306/test"
+                "mysql+mysqldb://scott:tiger@localhost:3306/test"
                 "?%s=%s" % (kwarg, value)
             )
         )
@@ -267,7 +327,7 @@ class DialectTest(fixtures.TestBase):
     def test_random_arg(self):
         dialect = testing.db.dialect
         kw = dialect.create_connect_args(
-            make_url("mysql://u:p@host/db?foo=true")
+            make_url("mysql+mysqldb://u:p@host/db?foo=true")
         )[1]
         eq_(kw["foo"], "true")
 
@@ -284,7 +344,6 @@ class DialectTest(fixtures.TestBase):
         ("utf8",),
     )
     def test_special_encodings(self, enc):
-
         eng = engines.testing_engine(
             options={"connect_args": {"charset": enc, "use_unicode": 0}}
         )
@@ -298,7 +357,6 @@ class DialectTest(fixtures.TestBase):
 
     @testing.only_on("mariadb+mariadbconnector")
     def test_mariadb_connector_special_encodings(self):
-
         eng = engines.testing_engine()
         conn = eng.connect()
 
@@ -308,7 +366,7 @@ class DialectTest(fixtures.TestBase):
 
 class ParseVersionTest(fixtures.TestBase):
     def test_mariadb_madness(self):
-        mysql_dialect = make_url("mysql://").get_dialect()()
+        mysql_dialect = make_url("mysql+mysqldb://").get_dialect()()
 
         is_(mysql_dialect.is_mariadb, False)
 
@@ -548,15 +606,3 @@ class ExecutionTest(fixtures.TestBase):
     def test_sysdate(self, connection):
         d = connection.execute(func.sysdate()).scalar()
         assert isinstance(d, datetime.datetime)
-
-
-class AutocommitTextTest(
-    test_deprecations.AutocommitKeywordFixture, fixtures.TestBase
-):
-    __only_on__ = "mysql", "mariadb"
-
-    def test_load_data(self):
-        self._test_keyword("LOAD DATA STUFF")
-
-    def test_replace(self):
-        self._test_keyword("REPLACE THING")
