@@ -1,4 +1,5 @@
 import importlib
+from inspect import signature
 import itertools
 import random
 
@@ -28,7 +29,6 @@ from sqlalchemy import tuple_
 from sqlalchemy import TypeDecorator
 from sqlalchemy import union
 from sqlalchemy import union_all
-from sqlalchemy import util
 from sqlalchemy import values
 from sqlalchemy.dialects import mysql
 from sqlalchemy.dialects import postgresql
@@ -43,10 +43,11 @@ from sqlalchemy.sql import roles
 from sqlalchemy.sql import True_
 from sqlalchemy.sql import type_coerce
 from sqlalchemy.sql import visitors
+from sqlalchemy.sql.annotation import Annotated
 from sqlalchemy.sql.base import HasCacheKey
+from sqlalchemy.sql.base import SingletonConstant
 from sqlalchemy.sql.elements import _label_reference
 from sqlalchemy.sql.elements import _textual_label_reference
-from sqlalchemy.sql.elements import Annotated
 from sqlalchemy.sql.elements import BindParameter
 from sqlalchemy.sql.elements import ClauseElement
 from sqlalchemy.sql.elements import ClauseList
@@ -62,10 +63,10 @@ from sqlalchemy.sql.lambdas import lambda_stmt
 from sqlalchemy.sql.lambdas import LambdaElement
 from sqlalchemy.sql.lambdas import LambdaOptions
 from sqlalchemy.sql.selectable import _OffsetLimitParam
-from sqlalchemy.sql.selectable import AliasedReturnsRows
 from sqlalchemy.sql.selectable import FromGrouping
 from sqlalchemy.sql.selectable import LABEL_STYLE_NONE
 from sqlalchemy.sql.selectable import LABEL_STYLE_TABLENAME_PLUS_COL
+from sqlalchemy.sql.selectable import NoInit
 from sqlalchemy.sql.selectable import Select
 from sqlalchemy.sql.selectable import Selectable
 from sqlalchemy.sql.selectable import SelectStatementGrouping
@@ -168,7 +169,7 @@ class MyType3(TypeDecorator):
         self.arg = arg
 
 
-class CoreFixtures(object):
+class CoreFixtures:
     # lambdas which return a tuple of ColumnElement objects.
     # must return at least two objects that should compare differently.
     # to test more varieties of "difference" additional objects can be added.
@@ -205,6 +206,43 @@ class CoreFixtures(object):
                 bindparam("bar", type_=String)
             ),
         ),
+        lambda: (
+            # test #11471
+            text("select * from table")
+            .columns(a=Integer())
+            .add_cte(table_b.select().cte()),
+            text("select * from table")
+            .columns(a=Integer())
+            .add_cte(table_b.select().where(table_b.c.a > 5).cte()),
+        ),
+        lambda: (
+            union(
+                select(table_a).where(table_a.c.a > 1),
+                select(table_a).where(table_a.c.a < 1),
+            ).add_cte(select(table_b).where(table_b.c.a > 1).cte("ttt")),
+            union(
+                select(table_a).where(table_a.c.a > 1),
+                select(table_a).where(table_a.c.a < 1),
+            ).add_cte(select(table_b).where(table_b.c.a < 1).cte("ttt")),
+            union(
+                select(table_a).where(table_a.c.a > 1),
+                select(table_a).where(table_a.c.a < 1),
+            )
+            .add_cte(select(table_b).where(table_b.c.a > 1).cte("ttt"))
+            ._annotate({"foo": "bar"}),
+        ),
+        lambda: (
+            union(
+                select(table_a).where(table_a.c.a > 1),
+                select(table_a).where(table_a.c.a < 1),
+            ).self_group(),
+            union(
+                select(table_a).where(table_a.c.a > 1),
+                select(table_a).where(table_a.c.a < 1),
+            )
+            .self_group()
+            ._annotate({"foo": "bar"}),
+        ),
         lambda: (
             literal(1).op("+")(literal(1)),
             literal(1).op("-")(literal(1)),
@@ -218,6 +256,8 @@ class CoreFixtures(object):
             column("q") == column("x"),
             column("q") == column("y"),
             column("z") == column("x"),
+            (column("z") == column("x")).self_group(),
+            (column("q") == column("x")).self_group(),
             column("z") + column("x"),
             column("z").op("foo")(column("x")),
             column("z").op("foo")(literal(1)),
@@ -376,6 +416,10 @@ class CoreFixtures(object):
         lambda: (table_a.c.a, table_b.c.a),
         lambda: (tuple_(1, 2), tuple_(3, 4)),
         lambda: (func.array_agg([1, 2]), func.array_agg([3, 4])),
+        lambda: (
+            func.aggregate_strings(table_a.c.b, ","),
+            func.aggregate_strings(table_b_like_a.c.b, ","),
+        ),
         lambda: (
             func.percentile_cont(0.5).within_group(table_a.c.a),
             func.percentile_cont(0.5).within_group(table_a.c.b),
@@ -463,6 +507,21 @@ class CoreFixtures(object):
             select(table_a.c.a)
             .where(table_a.c.b == 5)
             .with_for_update(nowait=True),
+            select(table_a.c.a)
+            .where(table_a.c.b == 5)
+            .with_for_update(nowait=True, skip_locked=True),
+            select(table_a.c.a)
+            .where(table_a.c.b == 5)
+            .with_for_update(nowait=True, read=True),
+            select(table_a.c.a)
+            .where(table_a.c.b == 5)
+            .with_for_update(of=table_a.c.a),
+            select(table_a.c.a)
+            .where(table_a.c.b == 5)
+            .with_for_update(of=table_a.c.b),
+            select(table_a.c.a)
+            .where(table_a.c.b == 5)
+            .with_for_update(nowait=True, key_share=True),
             select(table_a.c.a).where(table_a.c.b == 5).correlate(table_b),
             select(table_a.c.a)
             .where(table_a.c.b == 5)
@@ -641,6 +700,15 @@ class CoreFixtures(object):
             )
             .data([(1, "textA", 99), (2, "textB", 88)])
             ._annotate({"nocache": True}),
+            values(
+                column("mykey", Integer),
+                column("mytext", String),
+                column("myint", Integer),
+                name="myvalues",
+                literal_binds=True,
+            )
+            .data([(1, "textA", 99), (2, "textB", 88)])
+            ._annotate({"nocache": True}),
             values(
                 column("mykey", Integer),
                 column("mytext", String),
@@ -667,15 +735,62 @@ class CoreFixtures(object):
             ._annotate({"nocache": True}),
             # TODO: difference in type
             # values(
-            #    [
-            #        column("mykey", Integer),
-            #        column("mytext", Text),
-            #        column("myint", Integer),
-            #    ],
-            #    (1, "textA", 99),
-            #    (2, "textB", 88),
-            #    alias_name="myvalues",
-            # ),
+            #     column("mykey", Integer),
+            #     column("mytext", Text),
+            #     column("myint", Integer),
+            #     name="myvalues",
+            # )
+            # .data([(1, "textA", 99), (2, "textB", 88)])
+            # ._annotate({"nocache": True}),
+        ),
+        lambda: (
+            values(
+                column("mykey", Integer),
+                column("mytext", String),
+                column("myint", Integer),
+                name="myvalues",
+            )
+            .data([(1, "textA", 99), (2, "textB", 88)])
+            .scalar_values()
+            ._annotate({"nocache": True}),
+            values(
+                column("mykey", Integer),
+                column("mytext", String),
+                column("myint", Integer),
+                name="myvalues",
+                literal_binds=True,
+            )
+            .data([(1, "textA", 99), (2, "textB", 88)])
+            .scalar_values()
+            ._annotate({"nocache": True}),
+            values(
+                column("mykey", Integer),
+                column("mytext", String),
+                column("myint", Integer),
+                name="myvalues",
+            )
+            .data([(1, "textA", 89), (2, "textG", 88)])
+            .scalar_values()
+            ._annotate({"nocache": True}),
+            values(
+                column("mykey", Integer),
+                column("mynottext", String),
+                column("myint", Integer),
+                name="myvalues",
+            )
+            .data([(1, "textA", 99), (2, "textB", 88)])
+            .scalar_values()
+            ._annotate({"nocache": True}),
+            # TODO: difference in type
+            # values(
+            #     column("mykey", Integer),
+            #     column("mytext", Text),
+            #     column("myint", Integer),
+            #     name="myvalues",
+            # )
+            # .data([(1, "textA", 99), (2, "textB", 88)])
+            # .scalar_values()
+            # ._annotate({"nocache": True}),
         ),
         lambda: (
             select(table_a.c.a),
@@ -845,7 +960,6 @@ class CoreFixtures(object):
             return stmt
 
         def three():
-
             a1 = table_a.alias()
             a2 = table_a.alias()
             ex = exists().where(table_b.c.b == a1.c.a)
@@ -872,7 +986,6 @@ class CoreFixtures(object):
     fixtures.append(_complex_fixtures)
 
     def _statements_w_context_options_fixtures():
-
         return [
             select(table_a)._add_context_option(opt1, True),
             select(table_a)._add_context_option(opt1, 5),
@@ -921,7 +1034,6 @@ class CoreFixtures(object):
             return anon_col > 5
 
         def three():
-
             l1, l2 = table_a.c.a.label(None), table_a.c.b.label(None)
 
             stmt = select(table_a.c.a, table_a.c.b, l1, l2)
@@ -982,8 +1094,7 @@ class CoreFixtures(object):
             .where(table_b_b.c.c > 10),
         )
 
-    if util.py37:
-        fixtures.append(_update_dml_w_dicts)
+    fixtures.append(_update_dml_w_dicts)
 
     def _lambda_fixtures():
         def one():
@@ -1288,6 +1399,246 @@ class CacheKeyTest(fixtures.CacheKeyFixture, CoreFixtures, fixtures.TestBase):
         is_not(ck3, None)
 
 
+def all_hascachekey_subclasses(ignore_subclasses=()):
+    def find_subclasses(cls: type):
+        for s in class_hierarchy(cls):
+            if (
+                # class_hierarchy may return values that
+                # aren't subclasses of cls
+                not issubclass(s, cls)
+                or "_traverse_internals" not in s.__dict__
+                or any(issubclass(s, ignore) for ignore in ignore_subclasses)
+            ):
+                continue
+            yield s
+
+    return dict.fromkeys(find_subclasses(HasCacheKey))
+
+
+class HasCacheKeySubclass(fixtures.TestBase):
+    custom_traverse = {
+        "AnnotatedFunctionAsBinary": {
+            "sql_function",
+            "left_index",
+            "right_index",
+            "modifiers",
+            "_annotations",
+        },
+        "Annotatednext_value": {"sequence", "_annotations"},
+        "FunctionAsBinary": {
+            "sql_function",
+            "left_index",
+            "right_index",
+            "modifiers",
+        },
+        "next_value": {"sequence"},
+    }
+
+    ignore_keys = {
+        "AnnotatedColumn": {"dialect_options"},
+        "SelectStatementGrouping": {
+            "_independent_ctes",
+            "_independent_ctes_opts",
+        },
+    }
+
+    @testing.combinations(*all_hascachekey_subclasses())
+    def test_traverse_internals(self, cls: type):
+        super_traverse = {}
+        # ignore_super = self.ignore_super.get(cls.__name__, set())
+        for s in cls.mro()[1:]:
+            # if s.__name__ in ignore_super:
+            #     continue
+            if s.__name__ == "Executable":
+                continue
+            for attr in s.__dict__:
+                if not attr.endswith("_traverse_internals"):
+                    continue
+                for k, v in s.__dict__[attr]:
+                    if k not in super_traverse:
+                        super_traverse[k] = v
+        traverse_dict = dict(cls.__dict__["_traverse_internals"])
+        eq_(len(cls.__dict__["_traverse_internals"]), len(traverse_dict))
+        if cls.__name__ in self.custom_traverse:
+            eq_(traverse_dict.keys(), self.custom_traverse[cls.__name__])
+        else:
+            ignore = self.ignore_keys.get(cls.__name__, set())
+
+            left_keys = traverse_dict.keys() | ignore
+            is_true(
+                left_keys >= super_traverse.keys(),
+                f"{left_keys} >= {super_traverse.keys()} - missing: "
+                f"{super_traverse.keys() - left_keys} - ignored {ignore}",
+            )
+
+            subset = {
+                k: v for k, v in traverse_dict.items() if k in super_traverse
+            }
+            eq_(
+                subset,
+                {k: v for k, v in super_traverse.items() if k not in ignore},
+            )
+
+    # name -> (traverse names, init args)
+    custom_init = {
+        "BinaryExpression": (
+            {"right", "operator", "type", "negate", "modifiers", "left"},
+            {"right", "operator", "type_", "negate", "modifiers", "left"},
+        ),
+        "BindParameter": (
+            {"literal_execute", "type", "callable", "value", "key"},
+            {"required", "isoutparam", "literal_execute", "type_", "callable_"}
+            | {"unique", "expanding", "quote", "value", "key"},
+        ),
+        "Cast": ({"type", "clause"}, {"type_", "expression"}),
+        "ClauseList": (
+            {"clauses", "operator"},
+            {"group_contents", "group", "operator", "clauses"},
+        ),
+        "ColumnClause": (
+            {"is_literal", "type", "table", "name"},
+            {"type_", "is_literal", "text"},
+        ),
+        "ExpressionClauseList": (
+            {"clauses", "operator"},
+            {"type_", "operator", "clauses"},
+        ),
+        "FromStatement": (
+            {"_raw_columns", "_with_options", "element"}
+            | {"_propagate_attrs", "_with_context_options"},
+            {"element", "entities"},
+        ),
+        "FunctionAsBinary": (
+            {"modifiers", "sql_function", "right_index", "left_index"},
+            {"right_index", "left_index", "fn"},
+        ),
+        "FunctionElement": (
+            {"clause_expr", "_table_value_type", "_with_ordinality"},
+            {"clauses"},
+        ),
+        "Function": (
+            {"_table_value_type", "clause_expr", "_with_ordinality"}
+            | {"packagenames", "type", "name"},
+            {"type_", "packagenames", "name", "clauses"},
+        ),
+        "Label": ({"_element", "type", "name"}, {"type_", "element", "name"}),
+        "LambdaElement": (
+            {"_resolved"},
+            {"role", "opts", "apply_propagate_attrs", "fn"},
+        ),
+        "Load": (
+            {"propagate_to_loaders", "additional_source_entities"}
+            | {"path", "context"},
+            {"entity"},
+        ),
+        "LoaderCriteriaOption": (
+            {"where_criteria", "entity", "propagate_to_loaders"}
+            | {"root_entity", "include_aliases"},
+            {"where_criteria", "include_aliases", "propagate_to_loaders"}
+            | {"entity_or_base", "loader_only", "track_closure_variables"},
+        ),
+        "NullLambdaStatement": ({"_resolved"}, {"statement"}),
+        "ScalarFunctionColumn": (
+            {"type", "fn", "name"},
+            {"type_", "name", "fn"},
+        ),
+        "ScalarValues": (
+            {"_data", "_column_args", "literal_binds"},
+            {"columns", "data", "literal_binds"},
+        ),
+        "Select": (
+            {
+                "_having_criteria",
+                "_distinct",
+                "_group_by_clauses",
+                "_fetch_clause",
+                "_limit_clause",
+                "_label_style",
+                "_order_by_clauses",
+                "_raw_columns",
+                "_correlate_except",
+                "_statement_hints",
+                "_hints",
+                "_independent_ctes",
+                "_distinct_on",
+                "_with_context_options",
+                "_setup_joins",
+                "_suffixes",
+                "_memoized_select_entities",
+                "_for_update_arg",
+                "_prefixes",
+                "_propagate_attrs",
+                "_with_options",
+                "_independent_ctes_opts",
+                "_offset_clause",
+                "_correlate",
+                "_where_criteria",
+                "_annotations",
+                "_fetch_clause_options",
+                "_from_obj",
+            },
+            {"entities"},
+        ),
+        "TableValuedColumn": (
+            {"scalar_alias", "type", "name"},
+            {"type_", "scalar_alias"},
+        ),
+        "TableValueType": ({"_elements"}, {"elements"}),
+        "TextualSelect": (
+            {"column_args", "_annotations", "_independent_ctes"}
+            | {"element", "_independent_ctes_opts"},
+            {"positional", "columns", "text"},
+        ),
+        "Tuple": ({"clauses", "operator"}, {"clauses", "types"}),
+        "TypeClause": ({"type"}, {"type_"}),
+        "TypeCoerce": ({"type", "clause"}, {"type_", "expression"}),
+        "UnaryExpression": (
+            {"modifier", "element", "operator"},
+            {"operator", "wraps_column_expression"}
+            | {"type_", "modifier", "element"},
+        ),
+        "Values": (
+            {"_column_args", "literal_binds", "name", "_data"},
+            {"columns", "name", "literal_binds"},
+        ),
+        "_FrameClause": (
+            {"upper_integer_bind", "upper_type"}
+            | {"lower_type", "lower_integer_bind"},
+            {"range_"},
+        ),
+        "_MemoizedSelectEntities": (
+            {"_with_options", "_raw_columns", "_setup_joins"},
+            {"args"},
+        ),
+        "next_value": ({"sequence"}, {"seq"}),
+    }
+
+    @testing.combinations(
+        *all_hascachekey_subclasses(
+            ignore_subclasses=[Annotated, NoInit, SingletonConstant]
+        )
+    )
+    def test_init_args_in_traversal(self, cls: type):
+        sig = signature(cls.__init__)
+        init_args = set()
+        for p in sig.parameters.values():
+            if (
+                p.name == "self"
+                or p.name.startswith("_")
+                or p.kind in (p.VAR_KEYWORD,)
+            ):
+                continue
+            init_args.add(p.name)
+
+        names = {n for n, _ in cls.__dict__["_traverse_internals"]}
+        if cls.__name__ in self.custom_init:
+            traverse, inits = self.custom_init[cls.__name__]
+            eq_(names, traverse)
+            eq_(init_args, inits)
+        else:
+            is_true(names.issuperset(init_args), f"{names} : {init_args}")
+
+
 class CompareAndCopyTest(CoreFixtures, fixtures.TestBase):
     @classmethod
     def setup_test_class(cls):
@@ -1305,19 +1656,14 @@ class CompareAndCopyTest(CoreFixtures, fixtures.TestBase):
         """
         need = set(
             cls
-            for cls in class_hierarchy(ClauseElement)
-            if issubclass(cls, (ColumnElement, Selectable, LambdaElement))
-            and (
-                "__init__" in cls.__dict__
-                or issubclass(cls, AliasedReturnsRows)
+            for cls in all_hascachekey_subclasses(
+                ignore_subclasses=[Annotated, NoInit, SingletonConstant]
             )
-            and not issubclass(cls, (Annotated))
-            and cls.__module__.startswith("sqlalchemy.")
-            and "orm" not in cls.__module__
+            if "orm" not in cls.__module__
             and "compiler" not in cls.__module__
-            and "crud" not in cls.__module__
-            and "dialects" not in cls.__module__  # TODO: dialects?
-        ).difference({ColumnElement, UnaryExpression})
+            and "dialects" not in cls.__module__
+            and issubclass(cls, (ColumnElement, Selectable, LambdaElement))
+        )
 
         for fixture in self.fixtures + self.dont_compare_values_fixtures:
             case_a = fixture()
@@ -1346,9 +1692,8 @@ class CompareAndCopyTest(CoreFixtures, fixtures.TestBase):
                                 compare_annotations=True,
                                 compare_values=compare_values,
                             ),
-                            "%r != %r" % (case_a[a], case_b[b]),
+                            f"{case_a[a]!r} != {case_b[b]!r} (index {a} {b})",
                         )
-
                     else:
                         is_false(
                             case_a[a].compare(
@@ -1356,7 +1701,7 @@ class CompareAndCopyTest(CoreFixtures, fixtures.TestBase):
                                 compare_annotations=True,
                                 compare_values=compare_values,
                             ),
-                            "%r == %r" % (case_a[a], case_b[b]),
+                            f"{case_a[a]!r} == {case_b[b]!r} (index {a} {b})",
                         )
 
     def test_compare_col_identity(self):
@@ -1456,7 +1801,6 @@ class CompareClausesTest(fixtures.TestBase):
         )
 
     def test_compare_metadata_tables_annotations_two(self):
-
         t1 = Table("a", MetaData(), Column("q", Integer), Column("p", Integer))
         t2 = Table("a", MetaData(), Column("q", Integer), Column("p", Integer))
 
@@ -1490,7 +1834,6 @@ class CompareClausesTest(fixtures.TestBase):
         ne_(t3._generate_cache_key(), t4._generate_cache_key())
 
     def test_compare_comparison_associative(self):
-
         l1 = table_c.c.x == table_d.c.y
         l2 = table_d.c.y == table_c.c.x
         l3 = table_c.c.x == table_d.c.z
@@ -1509,7 +1852,6 @@ class CompareClausesTest(fixtures.TestBase):
         is_false(l1.compare(l3))
 
     def test_compare_clauselist_associative(self):
-
         l1 = and_(table_c.c.x == table_d.c.y, table_c.c.y == table_d.c.z)
 
         l2 = and_(table_c.c.y == table_d.c.z, table_c.c.x == table_d.c.y)
@@ -1521,7 +1863,6 @@ class CompareClausesTest(fixtures.TestBase):
         is_false(l1.compare(l3))
 
     def test_compare_clauselist_not_associative(self):
-
         l1 = ClauseList(
             table_c.c.x, table_c.c.y, table_d.c.y, operator=operators.sub
         )
@@ -1534,7 +1875,6 @@ class CompareClausesTest(fixtures.TestBase):
         is_false(l1.compare(l2))
 
     def test_compare_clauselist_assoc_different_operator(self):
-
         l1 = and_(table_c.c.x == table_d.c.y, table_c.c.y == table_d.c.z)
 
         l2 = or_(table_c.c.y == table_d.c.z, table_c.c.x == table_d.c.y)
@@ -1542,13 +1882,12 @@ class CompareClausesTest(fixtures.TestBase):
         is_false(l1.compare(l2))
 
     def test_compare_clauselist_not_assoc_different_operator(self):
-
         l1 = ClauseList(
             table_c.c.x, table_c.c.y, table_d.c.y, operator=operators.sub
         )
 
         l2 = ClauseList(
-            table_c.c.x, table_c.c.y, table_d.c.y, operator=operators.div
+            table_c.c.x, table_c.c.y, table_d.c.y, operator=operators.truediv
         )
 
         is_false(l1.compare(l2))
